import logging

from zope.app.container.interfaces import IObjectAddedEvent
from zope.app.container.interfaces import IObjectRemovedEvent
from zope.app.form.interfaces import ConversionError
from zope.component import adapter, createObject
from zope.event import notify
from zope.lifecycleevent import Attributes
from zope.lifecycleevent import ObjectCreatedEvent, ObjectModifiedEvent
from zope.lifecycleevent.interfaces import IObjectCreatedEvent
from zope.lifecycleevent.interfaces import IObjectModifiedEvent
from zope.security.proxy import removeSecurityProxy

from zope.i18nmessageid import MessageFactory
_ = MessageFactory("largeblue")

from event import PageStructureNeedsReindex
from interfaces import IPage, IPageContainer, IFile, ISelfIndexing
from interfaces import IPageStructureNeedsReindexEvent


def is_page_container(obj):
    return IPageContainer.providedBy(obj) and not IPage.providedBy(obj)


def get_page_container(obj):
    if is_page_container(obj):
        return obj
    return hasattr(obj, '__parent__') and get_page_container(obj.__parent__) or None


@adapter(IObjectCreatedEvent)
def handle_created(event):
    obj = event.object
    if IPage.providedBy(obj):
        if not obj.has_key('main_content.html'):
            # create an html file called main content
            html_file = createObject('largeblue.pages.File')
            html_file.contentType = 'text/html'
            html_file.data = '<div>%s</div>' % _('Under construction')
            html_file.__name__ = 'main_content.html'
            notify(
                ObjectCreatedEvent(html_file)
            )
            obj['main_content.html'] = html_file
        
    


@adapter(IObjectModifiedEvent)
def handle_modified(event):
    obj = event.object
    if IPage.providedBy(obj):
        logging.info('page modified %s' % obj.__name__)
        notify(
            PageStructureNeedsReindex(
                get_page_container(obj)
            )
        )
    elif IPageContainer.providedBy(obj):
        logging.info('page container modified %s' % obj.__name__)
        notify(
            PageStructureNeedsReindex(obj)
        )
    elif IFile.providedBy(obj):
        parent = removeSecurityProxy(obj.__parent__)
        notify(
            ObjectModifiedEvent(
                parent,
                Attributes(IPage, 'content')
            )
        )
    


@adapter(IObjectAddedEvent)
def handle_added(event):
    obj = event.object
    obj_name = event.newName
    container = event.newParent
    if IPageContainer.providedBy(container):
        notify(
            PageStructureNeedsReindex(
                get_page_container(container)
            )
        )
    


@adapter(IObjectRemovedEvent)
def handle_deleted(event):
    obj = event.object
    obj_name = event.oldName
    container = event.oldParent
    if IPageContainer.providedBy(container):
        notify(
            PageStructureNeedsReindex(
                get_page_container(container)
            )
        )
    


@adapter(IPageStructureNeedsReindexEvent)
def handle_reindex(event):
    target = event.obj
    if ISelfIndexing.providedBy(target):
        target.reindex()




import logging

from zope.app import zapi
from zope.app.container.btree import BTreeContainer
from zope.app.container.contained import Contained
from zope.app.file.file import File as BaseFile
from zope.component import adapts, getUtility, createObject
from zope.component.factory import Factory
from zope.event import notify
from zope.lifecycleevent import Attributes
from zope.lifecycleevent import ObjectCreatedEvent, ObjectModifiedEvent
from zope.interface import implements

from zope.i18nmessageid import MessageFactory
_ = MessageFactory("largeblue")

from bebop.ordering.interfaces import IOrdering

from interfaces import IPage, IBasePage, IPageContainer, IFile, ISelfIndexing
from interfaces import IContainedWithinAPageContainer, IContainedWithinAPage


class Page(BTreeContainer):
    implements(
        IBasePage,
        IPage,
        IPageContainer,
        IContainedWithinAPageContainer
    )
    __name__ = u'page'
    __parent__ = None
    title = u''
    flag = _(u'Static')
    
    def get_content(self):
        if self.has_key('main_content.html'):
            html_file = self['main_content.html']
            if IFile.providedBy(html_file):
                return html_file.data
        return u''
    
    def set_content(self, data):
        done = False
        if self.has_key('main_content.html'):
            html_file = self['main_content.html']
            if IFile.providedBy(html_file):
                html_file.data = data
                done = True
                notify(
                    ObjectModifiedEvent(
                        html_file,
                        Attributes(IFile, 'data')
                    )
                )
        if not done:
            html_file = createObject('largeblue.pages.File')
            html_file.contentType = 'text/html'
            html_file.__name__ = 'main_content.html'
            html_file.data = data
            notify(ObjectCreatedEvent(html_file))
            self['main_content.html'] = html_file
    
    content = property(get_content, set_content)


pageFactory = Factory(
    Page,
    title = _(u'Create Page'),
    description = u''
)

class PageContainer(BTreeContainer):
    implements(IPageContainer, ISelfIndexing)
    index = []
    def _recursively_collect(self, container):
        pages = []
        for k, v in IOrdering(container).items():
            page = v.context
            item = {
                'label': page.__name__,
                'title': page.title,
                'pages': self._recursively_collect(page)
            }
            pages.append(item)
        return pages
    
    def reindex(self):
        self.index = self._recursively_collect(self)
    


class File(BaseFile):
    implements(IFile, IContainedWithinAPage)
    __name__ = __parent__ = None


fileFactory = Factory(
    File,
    title = _(u'Create Sub Page Container'),
    description = u''
)


import re

from zope.schema import TextLine
from zope.schema.interfaces import ValidationError

from zope.i18nmessageid import MessageFactory
_ = MessageFactory("largeblue")


_valid_characters = re.compile(r"^[a-z0-9_\.]+$").match
_starts_with_lowercase_letter = re.compile(r"^[a-z]+").match


def _is_valid_obj_name(n):
    return _valid_characters(n) and _starts_with_lowercase_letter(n)



class InvalidObjectName(ValidationError):
    __doc__ = _('must only contain lower case letters, numbers, full stops \
        and underscores and must start with a letter.'
    )



class ObjectName(TextLine):
    def _validate(self, value):
        # normal textline validation
        super(ObjectName, self)._validate(value)
        # now test to see it it matches our requirements
        if _is_valid_obj_name:
            return ''
        raise InvalidObjectName()
    




import os.path

from zope.testing import doctest
from zope.app.testing import functional


ftesting_zcml = os.path.join(
    os.path.dirname(__file__), 
    'ftesting.zcml'
)

FunctionalLayer = functional.ZCMLLayer(
    ftesting_zcml, 
    __name__,
    'FunctionalLayer'
)

__docformat__ = "reStructuredText"


def FunctionalDocTestSuite(module=None, **kw):
    module = doctest._normalize_module(module)
    suite = functional.FunctionalDocTestSuite(module, **kw)
    suite.layer = FunctionalLayer
    return suite


def FunctionalDocFileSuite(path, **kw):
    suite = functional.FunctionalDocFileSuite(path, **kw)
    suite.layer = FunctionalLayer
    return suite


class BrowserTestCase(functional.BrowserTestCase):
    layer = FunctionalLayer



import unittest, os

import zope, transaction
import os.path
from zope.testing import doctest, doctestunit, module
from zope.app.testing.functional import FunctionalDocFileSuite, FunctionalDocTestSuite, ZCMLLayer

import bop
import testing

BebopWebDAVLayer = ZCMLLayer(
    os.path.join(os.path.split(__file__)[0], 'ftesting.zcml'),
    __name__, 'BebopWebDAVLayer', allow_teardown=True)


def setUp(test):
    module.setUp(test, 'bebop.webdav.readme_txt')

def tearDown(test):
    module.tearDown(test, 'bebop.webdav.readme_txt')


def test_suite():
    suite = unittest.TestSuite()
    
    globs = {'os': os,
            'os.path': os.path,
            'zope': zope,
            'transaction': transaction,
            'pprint': doctestunit.pprint,
            'printEvent': testing.printEvent,
            'bop': bop}
            
                      
    test = FunctionalDocFileSuite('README.txt',
            package='bebop.webdav',
            globs=globs,
            optionflags=doctest.NORMALIZE_WHITESPACE+doctest.ELLIPSIS)

    test.layer = BebopWebDAVLayer
    suite.addTests((test,))
        
    return suite

if __name__ == '__main__':
    unittest.main(defaultTest='test_suite')



from zope.interface import Interface, Attribute
from zope.schema import Choice, Field, List, Text, TextLine
from zope.app.container.constraints import ContainerTypesConstraint
from zope.app.container.constraints import ItemTypePrecondition
from zope.app.container.interfaces import IContained, IContainer
from zope.app.file.interfaces import IFile as IBaseFile

from zope.i18nmessageid import MessageFactory
_ = MessageFactory("largeblue")

from schema import ObjectName


class IBasePage(IContainer):
    __name__ = ObjectName(
        title = _(u"Object Name"),
        description = _(u"The name as it appears in the object's URL.  Must be lowercase letter and numbers, with no white spaces.  Full stop '.' and underscore '_' characters are allowed"),
        default = u"page",
        required = True
    )
    title = TextLine(
        title = _(u"Human Friendly Title"),
        description = _(u"The title of the page, usually displayed in the top bar of the web browser and possible on the page as a main heading"),
        default = u"",
        required = True
    )
    flag = Choice(
        title = _(u"Page Type"),
        description = _(u"This is a (hopefully flexible) flag used to determine how to render it.  Typically, if 'static', then the Main Content (edited below) is simply rendered"),
        required = True,
        default = _(u'Static'),
        values = [
            _(u'Static'),
            _(u'Template'),
            _(u'Special')
        ]
    )
    content = Text(
        title = _(u"Main Content"),
        description = _(u"The content of the page, usually rendered in the main content area of the page, or, if the page is dynamic or special, then it may still be pulled into a sub part of the page"),
        default = u"",
        required = False
    )


class IPageContainer(IContainer):
    def __setitem__(name, obj):
        pass
    
    __setitem__.precondition = ItemTypePrecondition(IBasePage)
    


class IFile(IBaseFile):
    pass


class IPage(IBasePage):
    def __setitem__(name, obj):
        pass
    __setitem__.precondition = ItemTypePrecondition(
        IBasePage, IFile
    )


class IContainedWithinAPageContainer(IContained):
    __parent__ = Field(
        constraint = ContainerTypesConstraint(
            IPageContainer
        )
    )


class IContainedWithinAPage(IContained):
    __parent__ = Field(
        constraint = ContainerTypesConstraint(
            IPage
        )
    )


class ISelfIndexing(Interface):
    index = List(
        title = _(u"Index"),
        description = _(u"Cached index, used here to store a copy of the contained pages, so it doesn't have to be created each time it's looked up"),
        default = [],
        required = False
    )
    def reindex(self):
        pass
    


class ISelfIndexingPageContainer(IPageContainer, ISelfIndexing):
    pass


class IPageStructureNeedsReindexEvent(Interface):
    obj = Attribute(_(u'Page'))



import unittest

import zope.testing.module

# these are used by setup
import transaction
from ZODB.interfaces import IConnection
from ZODB.tests.util import DB

from persistent.interfaces import IPersistent

from zope import component
import zope.component.interfaces
import zope.location.interfaces
import zope.annotation
from zope.app.testing import placelesssetup
from zope.app.keyreference.persistent import KeyReferenceToPersistent, connectionOfPersistent
from zope.app.folder import rootFolder
from zope.app.component.site import LocalSiteManager, SiteManagerAdapter
from zope.app.intid import IntIds
from zope.app.intid.interfaces import IIntIds
import zope.app.component.hooks

from testing import FunctionalDocFileSuite, FunctionalLayer

import bebop.ordering.interfaces

from largeblue.pages.order import ordering, orderable
from largeblue.pages.interfaces import IPage, IPageContainer
from largeblue.pages.page import pageFactory, fileFactory
from largeblue.pages import subscribers

__docformat__ = "reStructuredText"


def setup(test):
    placelesssetup.setUp()
    component.provideAdapter(
        KeyReferenceToPersistent, 
        adapts = (
            IPersistent,
        )
    )
    component.provideAdapter(
        SiteManagerAdapter, (
            zope.location.interfaces.ILocation,
        ),
        zope.component.interfaces.IComponentLookup
    )
    component.provideAdapter(
        connectionOfPersistent,
        adapts = (
            IPersistent,
        ),
        provides = IConnection
    )
    component.provideAdapter(
        factory = zope.annotation.attribute.AttributeAnnotations,
        adapts = (
            zope.annotation.interfaces.IAttributeAnnotatable,
        ),
        provides = zope.annotation.interfaces.IAnnotations
    )
    component.provideAdapter(
        factory = ordering.Ordering,
        adapts = (
            IPageContainer,
        )
    )
    component.provideAdapter(
        factory = ordering.Ordering,
        adapts = (
            IPageContainer,
        )
    )
    component.provideAdapter(
        factory = orderable.OrderableFactory
    )
    component.provideHandler(subscribers.handle_created)
    component.provideHandler(subscribers.handle_modified)
    component.provideHandler(subscribers.handle_added)
    component.provideHandler(subscribers.handle_deleted)
    component.provideHandler(subscribers.handle_reindex)
    test.globs['db'] = db = DB()
    test.globs['conn'] = conn = db.open()
    test.globs['root'] = root = conn.root()
    test.globs['app'] = app = root['app'] = rootFolder()
    app.setSiteManager(
        LocalSiteManager(app)
    )
    zope.app.component.hooks.setSite(app)
    zope.app.component.hooks.setHooks()
    sm = app.getSiteManager()
    sm['intids'] = IntIds()
    registry = zope.component.interfaces.IComponentRegistry(sm)
    registry.registerUtility(sm['intids'], IIntIds)
    registry.registerUtility(
        pageFactory,
        zope.component.interfaces.IFactory, 
        'largeblue.pages.Page'
    )
    registry.registerUtility(
        fileFactory,
        zope.component.interfaces.IFactory, 
        'largeblue.pages.File'
    )
    transaction.commit()

def tearDown(test):
    zope.app.component.hooks.resetHooks()
    zope.app.component.hooks.setSite()
    transaction.abort()
    test.globs['db'].close()
    placelesssetup.tearDown()


def READMESetUp(test):
    setup(test)
    zope.testing.module.setUp(
        test, 
        'largeblue.pages.README'
    )

def READMETearDown(test):
    tearDown(test)
    zope.testing.module.tearDown(test)


def test_suite():
    return FunctionalDocFileSuite(
        'README.txt',
        setUp = READMESetUp, 
        tearDown = READMETearDown
    )


if __name__ == '__main__':
    unittest.main(
        defaultTest = 'test_suite'
    )


from zope.app.form.interfaces import ConversionError
from zope.component import createObject
from zope.exceptions.interfaces import UserError
from zope.formlib.form import EditForm, AddForm, Fields, applyChanges

from zope.i18nmessageid import MessageFactory
_ = MessageFactory("largeblue")

from bebop.ordering.interfaces import IOrderable, IOrdering

from largeblue.order.browser.view import OrderingView as DefaultOrderingView
from largeblue.order.ordering import Ordering

from largeblue.pages.interfaces import IPage
from largeblue.pages.schema import InvalidObjectName, _is_valid_obj_name

from widget import WYSIWYG


class PageAddForm(AddForm):
    form_fields = Fields(IPage).select('__name__', 'title', 'flag')
    label = _(u'Add Page')
    def create(self, data):
        self.context.contentName = data.get('__name__', u'')
        obj = createObject(u'largeblue.pages.Page')
        applyChanges(obj, self.form_fields, data)
        return obj
    


class PageEditForm(EditForm):
    form_fields = Fields(IPage).select('title', 'content')
    form_fields['content'].custom_widget = WYSIWYG
    label = _(u"Edit Page")


class OrderingView(DefaultOrderingView):
    def renameObjects(self):
        """
          
          We override this method to manually make sure that the
          new name(s) is/are valid before renaming the page(s)
          
          
        """
        
        for name in self.request.get("new_value"):
            if not _is_valid_obj_name(name):
                raise UserError(
                    '%s: %s' % (
                        _(u'Object name invalid'),
                        InvalidObjectName.__doc__
                    )
                )
        super(OrderingView, self).renameObjects()
    



#!/usr/local/env/python
#############################################################################
# Name:         locking.py
# Purpose:      Convenience functions for WebDAV locks
# Maintainer:   Uwe Oestermeier
# Copyright:    (c) 2007 iwm-kmrc.de KMRC - Knowledge Media Research Center
# Licence:      GPL
#############################################################################


import zope.locking.interfaces
from zope.app.keyreference.interfaces import IKeyReference

def lock(obj):
    broker = zope.locking.interfaces.ITokenBroker(obj, None)
    if broker is not None:
        return broker.lock()

def get(obj):
    broker = zope.locking.interfaces.ITokenBroker(obj, None)
    if broker is not None:
        try:
            keyreference = IKeyReference(obj, None)
            if keyreference is not None:
                return broker.get()
        except zope.app.keyreference.interfaces.NotYet:
            pass

def unlock(obj):
    token = get(obj)
    if token is not None:
        return token.end()

def owner(obj):
    try:
        token = get(obj)
        if token is not None:
            return sorted(token.principal_ids)
    except TypeError:   # not lockable
        return None




def printEvent(event):
    obj = getattr(event, 'object', None)
    if obj is None :
        name = ''
    else :
        name = getattr(obj, '__name__', 'unnamed')
    print "Event log:", event.__class__.__name__, name



__docformat__ = 'restructuredtext'

from zope import interface
from zope import component

from zope.app.security.interfaces import IUnauthenticatedPrincipal
from zope.security.interfaces import Unauthorized
from zope.filerepresentation.interfaces import IReadDirectory

import zope.app.container.interfaces
import z3c.dav.propfind 
import z3c.dav.interfaces

MS_WEBFOLDER = 'Microsoft Data Access Internet Publishing Provider DAV'


class PROPFINDPATCH(z3c.dav.propfind.PROPFIND):
    """A patch for the PROPFIND method.
    
    This patch solves the MS WebFolder issue discussed at
    
    http://www.nabble.com/MS-Webfolder-WebDAV-Client:
    -Authentication-issue-with-handling-of-replies-to-PUT-requests-t3137698.html
       
    """

    interface.implements(z3c.dav.interfaces.IWebDAVMethod)
    component.adapts(interface.Interface, z3c.dav.interfaces.IWebDAVRequest)
    
    def PROPFIND(self):
        """Checks whether the request is from a MS WebFolder.
        
        Raises an authentication prompt in order to ensure that
        the objects are locked and writeable.
        """
        
        if self.request['HTTP_USER_AGENT'] == MS_WEBFOLDER:
            print "USER", self.request.principal.id
            if IReadDirectory(self.context, None) is None:
               # skips all directories: only files are locked with this patch
                if IUnauthenticatedPrincipal.providedBy(self.request.principal):
                    response = self.request.response
                    response.setHeader("MS-Author-Via", "DAV")
                    response.setHeader("WWW-Authenticate", 'Negotiate')
                                  #  'basic realm="Zope"')
                    print "raise Unauthorized", self.request.principal.id
                    raise Unauthorized
        
        return super(PROPFINDPATCH, self).PROPFIND()


from zope.interface import implements

from interfaces import IPageStructureNeedsReindexEvent


class PageStructureNeedsReindex(object):
    implements(IPageStructureNeedsReindexEvent)
    def __init__(self, page_obj):
        self.obj = page_obj
    




#!/usr/local/env/python
#############################################################################
# Name:         bootstrap.py
# Purpose:      Ensures utility for the use of WebDAV locking
# Maintainer:   Uwe Oestermeier
# Copyright:    (c) 2007 iwm-kmrc.de KMRC - Knowledge Media Research Center
# Licence:      GPL
#############################################################################

import transaction
import zope.app.appsetup
from zope.app.appsetup import bootstrap

import zope.locking.interfaces
import zope.locking.utility

def databaseOpenedEventHandler(event):
    """Subscriber to the IDataBaseOpenedEvent

    Create utilities which are required for shortcuts etc.
    """
    db, connection, root, root_folder = bootstrap.getInformationFromEvent(event)
    
    bootstrap.ensureUtility(root_folder,
        zope.locking.interfaces.ITokenUtility,
        'zope.locking.tokens',
        zope.locking.utility.TokenUtility,
        asObject=True)
     
    transaction.commit()
    connection.close()

      

